/*
    *
    * This file is a part of CoreTime.
    * A time related task manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QMessageBox>
#include <QCheckBox>
#include <QSettings>
#include <QFileDialog>
#include <QKeyEvent>
#include <QEvent>
#include <QDebug>

#include <cprime/infofunc.h>

#include "alarm.h"
#include "alarmitem.h"
#include "alarmschedulequeue.h"
#include "ui_alarm.h"


Alarm::Alarm(QWidget *parent) : QWidget(parent), ui(new Ui::Alarm)
{
    ui->setupUi(this);

    m_addAlarm = false;
    m_editAlarm = false;
    temp = nullptr;
    ui->config->setVisible(false);
    ui->m_edit->setEnabled(false);
    ui->m_remove->setEnabled(false);

    loadAlarmItems();

    connect(ui->m_add, &QPushButton::clicked, this, &Alarm::addAlarm);
    connect(ui->m_edit, &QPushButton::clicked, this, &Alarm::editAlarm);
    connect(ui->m_remove, &QPushButton::clicked, this, &Alarm::removeAlarm);
    qDebug() << "Alarm Widget initilized...";
}

Alarm::~Alarm()
{
    saveAlarmItems();
    delete ui;
}

bool Alarm::event(QEvent *event)
{
    if (event->type() == QEvent::KeyPress) {
        QKeyEvent *key = reinterpret_cast<QKeyEvent *>(event);

        if (!(key->modifiers() & (Qt::ControlModifier | Qt::AltModifier))) {
            switch (key->key()) {
                case Qt::Key_Insert:
                    addAlarm();
                    event->accept();
                    return true;

                case Qt::Key_Delete:
                    removeAlarm();
                    event->accept();
                    return true;
            }
        }
    }

    return QWidget::event(event);
}

void Alarm::changeState(bool change)
{
    ui->config->setVisible(change);
    ui->m_alarmList->setEnabled(!change);
    ui->controlBar->setEnabled(!change);
    ui->config->setFocus();
}

void Alarm::configToAlarmItem(AlarmItem *item)
{
    item->setReminderText(ui->m_reminderText->text());
    item->setStartTime(QTime(ui->m_startTime->time().hour(), ui->m_startTime->time().minute()));
    item->setSnoozeTime(QTime(0, 1)); // Set snooze for 1 minute
    item->setRepeat(ui->m_weekdays->isChecked());
    item->setWeekdayEnable(AlarmItem::Monday, ui->m_mon->isChecked());
    item->setWeekdayEnable(AlarmItem::Tuesday, ui->m_tue->isChecked());
    item->setWeekdayEnable(AlarmItem::Wednesday, ui->m_wed->isChecked());
    item->setWeekdayEnable(AlarmItem::Thursday, ui->m_thu->isChecked());
    item->setWeekdayEnable(AlarmItem::Friday, ui->m_fri->isChecked());
    item->setWeekdayEnable(AlarmItem::Saturday, ui->m_sat->isChecked());
    item->setWeekdayEnable(AlarmItem::Sunday, ui->m_sun->isChecked());

    AlarmItem::AlarmType alarmType;

    if (ui->m_weekdays->isChecked()) {
        alarmType = AlarmItem::Repeat;
    } else {
        alarmType = AlarmItem::Once;
    }

    item->setAlarmType(alarmType);

    item->setIsCustomSound(ui->m_customSound->isChecked());
    item->setCustomSoundFile(ui->m_soundPath->text());
}

void Alarm::loadAlarmItems()
{
    QSettings *settings = new QSettings("coreapps", "alarm");

    int count = settings->value("AlarmCount").toInt();

    for (int i = 0; i < count; ++i) {
        AlarmItem *aItem = new AlarmItem;

        aItem->loadAlarm(i);

        QListWidgetItem *item = new QListWidgetItem;
        item->setSizeHint(QSize(aItem->sizeHint()));

        ui->m_alarmList->addItem(item);
        ui->m_alarmList->setItemWidget(item, aItem);

        m_alarmQueue.add(aItem);

        connect(aItem, &AlarmItem::alarmEnableToggled, this, &Alarm::alarmItemToggled);
    }
}

void Alarm::saveAlarmItems()
{
    qDebug() << "Save Called...";
    QSettings *settings = new QSettings("coreapps", "alarm");

    int count = ui->m_alarmList->count();
    int count2 = settings->value("AlarmCount").toInt();

    if (count == count2) {
        return;
    }

    settings->setValue("AlarmCount", count);

    int i;

    for (i = 0; i < count; i++) {
        AlarmItem *item = static_cast<AlarmItem *>(ui->m_alarmList->itemWidget(ui->m_alarmList->item(i)));
        item->saveAlarm(i);
    }

    if (count > count2) {
        return;
    }

    for (int j = i; j < count2; j++) {
        AlarmItem *item = static_cast<AlarmItem *>(ui->m_alarmList->itemWidget(ui->m_alarmList->item(i)));
        item->deleteAlarm(i);
    }
}

void Alarm::alarmItemToggled(bool enable)
{
    if (sender()) {
        if (enable) {
            m_alarmQueue.modify(qobject_cast<AlarmItem *>(sender()));
        }

        saveAlarmItems();
    }
}

//////////////////////////////////// Config ///////////////////////////////////////////
void Alarm::initializeConfig()
{
    ui->m_reminderText->setText("Alarm");
    ui->m_startTime->setTime(QTime::currentTime());

    ui->m_weekdays->setChecked(false);
    ui->m_mon->setChecked(false);
    ui->m_tue->setChecked(false);
    ui->m_wed->setChecked(false);
    ui->m_thu->setChecked(false);
    ui->m_fri->setChecked(false);
    ui->m_sat->setChecked(false);
    ui->m_sun->setChecked(false);

    ui->m_customSound->setChecked(false);
    ui->m_soundPath->clear();
}

bool Alarm::checkConfig()
{
    QString errorMsg;
    QWidget *errorWidget = nullptr;

    if (ui->m_reminderText->text().count() == 0) {
//        errorMsg = "Please specify an alarm name.";
//        errorWidget = ui->m_reminderText;
    } else if (ui->m_customSound->isChecked() && (ui->m_soundPath->text().count() == 0)) {
        errorMsg = "Please specify a sound file path";
        errorWidget = ui->m_soundPath;
    } else if (ui->m_weekdays->isChecked()) {
        bool anyChecked = false;
        QObjectList objL = ui->m_weekdays->children();

        foreach (QObject *obj, objL) {
            QCheckBox *c = static_cast<QCheckBox *>(obj);

            if (c->isChecked()) {
                anyChecked = true;
            }
        }

        if (!anyChecked) {
            errorMsg = "Please select any weekday for repeat";
            errorWidget = ui->m_weekdays;
        }
    }

    if (errorWidget) {
        QMessageBox msgBox;
        msgBox.setText(errorMsg);
        msgBox.exec();

        errorWidget->setFocus();

        return false;
    }

    return true;
}

void Alarm::on_m_weekdays_toggled(bool arg1)
{
    if (arg1) {
        ui->m_mon->setChecked(arg1);
    } else {
        QObjectList objL = ui->m_weekdays->children();

        foreach (QObject *obj, objL) {
            QPushButton *c = dynamic_cast<QPushButton *>(obj);

            if (c) {
                c->setChecked(false);
            }
        }
    }
}

void Alarm::on_m_browse_clicked()
{
    QStringList filters;
    filters << ("MP3 files (*.mp3)");
    filters << ("OGG files (*.ogg");
    filters << ("WAV files (*.wav)");
    filters << ("All files (*)");

    QFileDialog fileDlg(this);
    fileDlg.setNameFilters(filters);

    if (fileDlg.exec() == QDialog::Accepted) {
        QString soundFile(QDir::toNativeSeparators(fileDlg.selectedFiles().first()));
        ui->m_soundPath->setText(soundFile);
    }
}

void Alarm::on_m_ok_clicked()
{
    bool check = checkConfig();

    if (!check) {
        qDebug() << "Problem at config...";
        return;
    }

    if (m_addAlarm && !m_editAlarm) {
        AlarmItem *aItem = new AlarmItem;

        configToAlarmItem(aItem);

        aItem->setAlarmEnable(true);

        QListWidgetItem *item = new QListWidgetItem;
        item->setSizeHint(QSize(aItem->sizeHint()));

        ui->m_alarmList->addItem(item);
        ui->m_alarmList->setItemWidget(item, aItem);

        connect(aItem, &AlarmItem::alarmEnableToggled, this, &Alarm::alarmItemToggled);

        m_alarmQueue.add(aItem);
    } else if (!m_addAlarm && m_editAlarm) {
        configToAlarmItem(temp);
        m_alarmQueue.modify(temp);
        temp = nullptr;
    }

    saveAlarmItems();

    changeState(false);
    ui->controlBar->setVisible(true);

    CPrime::InfoFunc::messageEngine("Alarm Added", CPrime::MessageType::Info, this);
}

void Alarm::on_m_cancel_clicked()
{
    changeState(false);
    ui->controlBar->setVisible(true);
}
//////////////////////////////////////////////////////////////////////////////////

void Alarm::addAlarm()
{
    ui->controlBar->setVisible(false);
    m_addAlarm = true;
    m_editAlarm = false;
    changeState(true);
    initializeConfig();
}

void Alarm::editAlarm()
{
    m_addAlarm = false;
    m_editAlarm = true;

    QListWidgetItem *item;

    item = ui->m_alarmList->currentItem();

    if (!item) {
        return;
    }

    changeState(true);
    AlarmItem *aItem = static_cast<AlarmItem *>(ui->m_alarmList->itemWidget(item));
    ui->m_reminderText->setText(aItem->getReminderText());
    ui->m_startTime->setTime(aItem->getStartTime());
    ui->m_weekdays->setChecked(aItem->getAlarmType() == AlarmItem::Repeat);
    ui->m_mon->setChecked(aItem->getIsWeekdayEnable(AlarmItem::Monday));
    ui->m_tue->setChecked(aItem->getIsWeekdayEnable(AlarmItem::Tuesday));
    ui->m_wed->setChecked(aItem->getIsWeekdayEnable(AlarmItem::Wednesday));
    ui->m_thu->setChecked(aItem->getIsWeekdayEnable(AlarmItem::Thursday));
    ui->m_fri->setChecked(aItem->getIsWeekdayEnable(AlarmItem::Friday));
    ui->m_sat->setChecked(aItem->getIsWeekdayEnable(AlarmItem::Saturday));
    ui->m_sun->setChecked(aItem->getIsWeekdayEnable(AlarmItem::Sunday));
    ui->m_customSound->setChecked(aItem->getIsCustomSound());
    ui->m_soundPath->setText(aItem->getCustomSoundFile());
    temp = aItem;
    aItem->setAlarmEnable(true);
}

void Alarm::removeAlarm()
{
    QListWidgetItem *item;
    item = ui->m_alarmList->currentItem();

    if (!item) {
        return;
    }

    AlarmItem *aItem = static_cast<AlarmItem *>(ui->m_alarmList->itemWidget(item));
    m_alarmQueue.remove(aItem);
    ui->m_alarmList->removeItemWidget(item);
    aItem->disconnect();

    delete aItem;
    delete item;

    saveAlarmItems();
}


void Alarm::on_m_alarmList_itemSelectionChanged()
{
    if (ui->m_alarmList->currentItem()) {
        ui->m_remove->setEnabled(true);
        ui->m_edit->setEnabled(true);
    } else {
        ui->m_remove->setEnabled(false);
        ui->m_edit->setEnabled(false);
    }
}

void Alarm::on_chkReminderText_toggled(bool checked)
{
    ui->m_reminderText->setEnabled(checked);

    if (checked) {
        ui->m_reminderText->setText("Alarm");
    } else {
        ui->m_reminderText->setText("");
    }
}
