/*
    *
    * This file is a part of CoreTime.
    * A time related task manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QCheckBox>
#include <QLabel>
#include <QHBoxLayout>
#include <QSettings>

#include "alarmitem.h"


AlarmItem::AlarmItem(QWidget *parent) : QWidget(parent)
{
    m_alarmCheck     = new QCheckBox(this);
    m_startTimeLabel = new QLabel(this);
    m_alarmWeekday   = new QLabel(this);

    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->addWidget(m_alarmCheck);
    mainLayout->addWidget(m_startTimeLabel);
    mainLayout->addWidget(m_alarmWeekday);

    setLayout(mainLayout);

    connect(m_alarmCheck, &QCheckBox::toggled, this, &AlarmItem::alarmEnableToggled);
}

AlarmItem::~AlarmItem()
{

}

QTime AlarmItem::getStartTime() const
{
    return m_startTime;
}

void AlarmItem::setStartTime(const QTime &startTime)
{
    m_startTime = startTime;
    m_startTimeLabel->setText(m_startTime.toString("HH:mm"));
}

bool AlarmItem::getIsAlarmEnable() const
{
    return m_alarmCheck->isChecked();
}

void AlarmItem::setAlarmEnable(bool enable)
{
    m_alarmCheck->setChecked(enable);
}

QString AlarmItem::getReminderText() const
{
    return m_reminderText;
}

void AlarmItem::setReminderText(const QString &name)
{
    m_reminderText = name;
    m_alarmCheck->setText(m_reminderText);
}

QTime AlarmItem::getSnoozeTime() const
{
    return m_snoozeTime;
}

void AlarmItem::setSnoozeTime(const QTime &snoozeTime)
{
    m_snoozeTime = snoozeTime;
}

AlarmItem::AlarmType AlarmItem::getAlarmType() const
{
    return m_alarmType;
}

void AlarmItem::setAlarmType(const AlarmItem::AlarmType &t)
{
    m_alarmType = t;

    switch (m_alarmType) {
        case Once: {
            m_alarmWeekday->setText("Once");
            break;
        }

        case Repeat: {
            m_alarmWeekday->setText(weekdayToString());
            break;
        }

        case Snoozed: {
            m_alarmWeekday->setText(QString("Snoozed for %1 minute(s)").arg(getSnoozeTime().toString("m")));
        }
    }
}

bool AlarmItem::getIsRepeat() const
{
    return m_repeat;
}

void AlarmItem::setRepeat(bool repeat)
{
    m_repeat = repeat;
}

bool AlarmItem::getIsWeekdayEnable(AlarmItem::WeekDay w) const
{
    return m_weekdays.value(w);
}

void AlarmItem::setWeekdayEnable(AlarmItem::WeekDay w, bool enable)
{
    m_weekdays.insert(w, enable);
}

bool AlarmItem::getIsCustomSound() const
{
    return m_isCustomSound;
}

void AlarmItem::setIsCustomSound(bool enable)
{
    m_isCustomSound = enable;
}

QString AlarmItem::getCustomSoundFile() const
{
    return m_customSoundFile;
}

void AlarmItem::setCustomSoundFile(const QString &file)
{
    m_customSoundFile = file;
}

void AlarmItem::loadAlarm(int index)
{
    QString alarmId(QString("Alarm%1").arg(index));
    QSettings *settings = new QSettings("coreapps", "alarm");

    settings->beginGroup(alarmId);
    setAlarmEnable(settings->value("AlarmEnable").toBool());
    setReminderText(settings->value("ReminderText").toString());
    setStartTime(settings->value("StartTime").toTime());
    setSnoozeTime(settings->value("SnoozeTime").toTime());

    setIsCustomSound(settings->value("CustomSound").toBool());
    setCustomSoundFile(settings->value("CustomSoundFile").toString());

    setRepeat(settings->value("Repeat").toBool());
    settings->beginGroup("Weekdays");

    for (int i = 0; i < 7; i++) {
        setWeekdayEnable(intToWeekday(i + 1), settings->value(QString::number(i + 1)).toBool());
    }

    settings->endGroup();

    setAlarmType(static_cast<AlarmType>(settings->value("AlarmType").toInt()));
    settings->endGroup();
}

void AlarmItem::deleteAlarm(int index)
{
    QString alarmId(QString("Alarm%1").arg(index));
    QSettings *settings = new QSettings("coreapps", "alarm");
    settings->remove(alarmId);
    settings->sync();
}

void AlarmItem::saveAlarm(int index)
{
    QString alarmId(QString("Alarm%1").arg(index));
    QSettings *settings = new QSettings("coreapps", "alarm");

    settings->beginGroup(alarmId);
    settings->setValue("AlarmEnable", getIsAlarmEnable());
    settings->setValue("ReminderText", getReminderText());
    settings->setValue("StartTime", getStartTime());
    settings->setValue("SnoozeTime", getSnoozeTime());

    settings->setValue("CustomSound", getIsCustomSound());
    settings->setValue("CustomSoundFile", getCustomSoundFile());

    settings->setValue("Repeat", getIsRepeat());
    settings->beginGroup("Weekdays");

    for (int i = 0; i < 7; i++) {
        settings->setValue(QString::number(i + 1), getIsWeekdayEnable(intToWeekday(i + 1)));
    }

    settings->endGroup();

    settings->setValue("AlarmType", getAlarmType());
    settings->endGroup();
    settings->sync();
}

QString AlarmItem::weekdayToString() const
{
    QString s;

    if (getIsWeekdayEnable(Monday)) {
        s.append("Mon").append(" ");
    }

    if (getIsWeekdayEnable(Tuesday)) {
        s.append("Tue").append(" ");
    }

    if (getIsWeekdayEnable(Wednesday)) {
        s.append("Wed").append(" ");
    }

    if (getIsWeekdayEnable(Thursday)) {
        s.append("Thu").append(" ");
    }

    if (getIsWeekdayEnable(Friday)) {
        s.append("Fri").append(" ");
    }

    if (getIsWeekdayEnable(Saturday)) {
        s.append("Sat").append(" ");
    }

    if (getIsWeekdayEnable(Sunday)) {
        s.append("Sun").append(" ");
    }

    return s;
}

AlarmItem::WeekDay AlarmItem::intToWeekday(int w)
{
    return static_cast<WeekDay>(w - 1);
}

