/*
    *
    * This file is a part of CoreTime.
    * A time related task manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QFile>
#include <QDir>
#include <QDebug>
#include <QDateTime>
#include <QDialog>
#include <QLabel>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QMediaPlayer>

#include "alarmschedulequeue.h"
#include "alarmitem.h"


QString defaultSound = QDir::tempPath() + "/sound.ogg";

AlarmScheduleQueue::AlarmScheduleQueue(QObject *parent) : QObject(parent)
{
    connect(&m_timer, &QTimer::timeout, this, &AlarmScheduleQueue::timerTimeout);
    m_timer.start(1000);
}

AlarmScheduleQueue::~AlarmScheduleQueue()
{
    delExtracted();
}

void AlarmScheduleQueue::add(const AlarmItem *a)
{
    m_alarmMap.insert(a, findNextAlarm(a, QDateTime(QDate::currentDate(), a->getStartTime()), true));
}

void AlarmScheduleQueue::remove(const AlarmItem *a)
{
    m_alarmMap.remove(a);
}

void AlarmScheduleQueue::modify(const AlarmItem *a)
{
    add(a);
}

QDateTime AlarmScheduleQueue::findNextAlarm(const AlarmItem *a, const QDateTime &dt, bool inclusive)
{
    QDateTime curr(QDateTime::currentDateTime());
    curr.setTime(QTime(curr.time().hour(), curr.time().minute()/*, curr.time().second()*/));

    QDateTime next(dt);

    if (a->getAlarmType() == AlarmItem::Repeat) {
        if (inclusive && a->getIsWeekdayEnable(a->intToWeekday(next.date().dayOfWeek())) && next >= curr) {
            return next;
        }

        for (int i = 0; i < 7; i++) {
            int dayOfWeek = next.addDays(i + 1).date().dayOfWeek();

            if (a->getIsWeekdayEnable(a->intToWeekday(dayOfWeek))) {
                next = next.addDays(i + 1);
                break;
            }
        }
    } else if (a->getAlarmType() == AlarmItem::Snoozed) {
        if (inclusive && next >= curr) {
            return next;
        }

        while (next <= curr) {
            if (inclusive && (next == curr)) {
                break;
            }

            next = next.addSecs(a->getSnoozeTime().minute() * 60);
        }
    }

    return next;
}

void AlarmScheduleQueue::alarm(const AlarmItem *a, const QDateTime &dt)
{
    QMediaPlayer *media = new QMediaPlayer(this);

    if (a->getIsCustomSound()) {
        media->setMedia(QUrl::fromLocalFile(a->getCustomSoundFile()));
    } else {
        // Extract default sound and play
        if (extractAudio()) {
            qDebug() << "Default Audio extracted...";
        } else {
            qDebug() << "No need to extract...";
        }

        media->setMedia(QUrl::fromLocalFile(defaultSound));
    }

    media->setVolume(80);
    media->setAudioRole(QAudio::AlarmRole);
    connect(media, &QMediaPlayer::mediaStatusChanged, [a, media, this](QMediaPlayer::MediaStatus state) {
        qDebug() << state;

        if (state == QMediaPlayer::EndOfMedia) {
            if (a->getIsCustomSound()) {
                media->setMedia(QUrl::fromLocalFile(a->getCustomSoundFile()));
            } else {
                // Extract default sound and play
                if (extractAudio()) {
                    qDebug() << "Default Audio extracted...";
                } else {
                    qDebug() << "No need to extract...";
                }

                media->setMedia(QUrl::fromLocalFile(defaultSound));
            }

            media->play();
        }
    });
    media->play();

    QDialog *sndlg = new QDialog;
    sndlg ->setAttribute(Qt::WA_DeleteOnClose);
    sndlg->setWindowTitle("Alarm & Reminder");
    sndlg->setWindowIcon(QIcon(":/icons/coretime.svg"));
    sndlg->resize(200, 80);

    QLabel *reminderText = new QLabel;
    reminderText->setText(a->getReminderText());
    reminderText->setAlignment(Qt::AlignCenter);

    QLabel *time = new QLabel;
    time->setText(QString("%1 \n %2").arg(dt.toString("hh:mm AP")).arg(QTime::currentTime().toString("hh:mm:ss AP")));
    time->setAlignment(Qt::AlignCenter);

    QPushButton *btnSn = new QPushButton;
    btnSn->setText("Snooze");

    QPushButton *btnCl = new QPushButton;
    btnCl->setText("Close");

    QHBoxLayout *btnLay = new QHBoxLayout;
    btnLay->addWidget(btnSn);
    btnLay->addWidget(btnCl);

    QVBoxLayout *mainLay = new QVBoxLayout;
    mainLay->addWidget(time);
    mainLay->addWidget(reminderText);
    mainLay->addLayout(btnLay);

    sndlg->setLayout(mainLay);
    sndlg->show();
    sndlg->activateWindow();
    sndlg->raise();

    QTimer *update = new QTimer;
    connect(update, &QTimer::timeout, [time, dt]() {
        time->setText(QString("%1 \n%2").arg(dt.toString("hh:mm AP")).arg(QTime::currentTime().toString("hh:mm:ss AP")));
    });
    update->start(1000);

    connect(sndlg, &QDialog::destroyed, [media, update]() {
        update->stop();
        update->deleteLater();
        media->deleteLater();
    });
    connect(btnSn, &QPushButton::clicked, [sndlg/*, a*/]() {
        //        a->setAlarmType(AlarmItem::Snoozed);
        //        modify(a);
        sndlg->close();
    });
    connect(btnCl, &QPushButton::clicked, [sndlg/*, a*/]() {
        //        if (a->getAlarmType() == )
        sndlg->close();
    });
}

bool AlarmScheduleQueue::extractAudio()
{
    if (QFile::copy(":/sound/icons/other/sound.ogg", defaultSound)) {
        return true;
    }

    return false;
}

bool AlarmScheduleQueue::delExtracted()
{
    return QFile::remove(defaultSound);
}

void AlarmScheduleQueue::timerTimeout()
{
    QDateTime curr(QDateTime::currentDateTime());
    QMap<const AlarmItem *, QDateTime> bellMap;
    QMapIterator<const AlarmItem *, QDateTime> it(m_alarmMap);

    while (it.hasNext()) {
        it.next();

        const AlarmItem *a(it.key());
        QDateTime dt(it.value());

        if (dt.date() == curr.date() && dt.time().hour() == curr.time().hour() &&
                dt.time().minute() == curr.time().minute() /*&& dt.time().second() == curr.time().second()*/) {
            bellMap.insert(a, dt);
        }
    }

    it = QMapIterator<const AlarmItem *, QDateTime>(bellMap);

    while (it.hasNext()) {
        it.next();

        const AlarmItem *a(it.key());
        QDateTime dt(it.value());

        if (a->getIsAlarmEnable()) {
            alarm(a, curr);
        }

        if (a->getAlarmType() == AlarmItem::Once) {
            remove(a);
            const_cast<AlarmItem *>(a)->setAlarmEnable(false);
        } else {
            m_alarmMap.insert(a, findNextAlarm(a, dt));
        }

    }
}
