/*
    *
    * This file is a part of CoreTime.
    * A time related task manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <cprime/themefunc.h>
#include <cprime/infofunc.h>

#include "alarm.h"
#include "stopwatch.h"
#include "coretime.h"
#include "ui_coretime.h"


coretime::coretime(QWidget *parent) : QWidget(parent), ui(new Ui::coretime)
{
    ui->setupUi(this);

    loadSettings();
    startSetup();

    on_clock_clicked();

    initializeClock();
    initializeAlarm();
    initializeStopwatch();
    initializeTimer();
}

coretime::~coretime()
{
    delete ui;
}

/**
 * @brief Setup ui elements
 */
void coretime::startSetup()
{
    // set stylesheet from style.qrc
    ui->sideView->setStyleSheet(CPrime::ThemeFunc::getSideViewStyleSheet());
    ui->sideView->setFixedWidth(sideViewSize);

    // set window size
    if (touch) {
        this->setWindowState(Qt::WindowMaximized);
    } else {
        this->resize(800, 500);
    }

    // all toolbuttons icon size in sideView
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(sideViewIconSize);
        }
    }

    on_appTitle_clicked();
}

/**
 * @brief Loads application settings
 */
void coretime::loadSettings()
{
    // Sideview is visible in default
    sideView = 1;

    sideViewSize = sm->value("CoreApps", "SideViewSize");
    sideViewIconSize = sm->value("CoreApps", "SideViewIconSize");
    touch = sm->value("CoreApps", "TouchMode");
}

void coretime::closeEvent(QCloseEvent *event)
{
    event->ignore();
    qApp->quit();
    event->accept();
}

void coretime::initializeClock()
{
    ui->clockText->setText(QTime::currentTime().toString("hh:mm:ss AP") + "\n" + QDate::currentDate().toString("dddd dd MMMM yyyy"));
    ui->calendarWid->showToday();
    clockTimer = new QTimer(this);
    connect(clockTimer, &QTimer::timeout, this, [&]() {
        ui->clockText->setText(QTime::currentTime().toString("hh:mm:ss AP") + "\n" + QDate::currentDate().toString("dddd dd MMMM yyyy"));
    });
    clockTimer->start(1000);
}

void coretime::initializeAlarm()
{
    ui->alarmLayout->addWidget(new Alarm(this));
}

void coretime::initializeStopwatch()
{
    ui->lapText->clear();
    ui->startandpause->setChecked(false);
    st = new StopWatch(this);
    connect(st, &StopWatch::time, this, &coretime::updateStopwatchTime);
}

void coretime::initializeTimer()
{
    ui->timerDuration->setVisible(false);
    forTimer = new QTimer(this);
    connect(forTimer, &QTimer::timeout, this, &coretime::updateTimer);
}

//======================Stopwatch=====================Start==================================

void coretime::on_startandpause_clicked(bool checked)
{
    if (checked) {
        st->start();
        ui->startandpause->setText("Pause");
        ui->lapandreset->setText("Lap");
    } else {
        st->pause();
        ui->startandpause->setText("Resume");
        ui->lapandreset->setText("Reset");
    }
}

void coretime::on_lapandreset_clicked()
{
    if (ui->lapandreset->text() == "Reset") {
        st->reset();
        ui->startandpause->setText("Start");
        ui->sw->setText("00:00:00.000");
        ui->lapText->clear();
    } else {
        int count = 1;

        if (ui->lapText->document()->toPlainText().count()) {
            count += ui->lapText->document()->lineCount();
        }

        ui->lapText->appendPlainText("#" + QString::number(count) + "\t\t" + ui->sw->text());
    }
}

void coretime::updateStopwatchTime(int time)
{
    QTime t;
    t.setHMS(time / MSECS_PER_HOUR,
             (time % MSECS_PER_HOUR) / MSECS_PER_MIN,
             (time / MSECS_PER_SEC) % 60,
             time % MSECS_PER_SEC);
    ui->sw->setText(/*st->currTime.getSTime()*//*st->sTime*/ t.toString("hh:mm:ss:zzz"));
}

//======================Stopwatch=====================End==================================

//======================Timer=======================Start==================================

void coretime::updateTimer()
{
    if (!(ui->timerText->text() == "00:00:00")) {
        QTime time = QTime::fromString(ui->timerText->text(), "hh:mm:ss");
        qint64 total = ((time.hour() * 60) * 60) + (time.minute() * 60) + (time.second() - 1);
        int h = static_cast<int>(total / 60 / 60);
        int m = static_cast<int>((total / 60) - (h * 60));
        int s = static_cast<int>(total - ((m + (h * 60)) * 60));
        time.setHMS(h, m, s);
        ui->timerText->setText(time.toString("hh:mm:ss"));
    } else {
        forTimer->stop();
        on_resetTimer_clicked();
        CPrime::InfoFunc::messageEngine("Timer Finished.", CPrime::MessageType::Info, this);
    }
}

void coretime::on_addTimer_clicked()
{
    if (ui->setTimer->text() == "00:00:00") {
        return;
    }

    ui->timerText->setText(ui->setTimer->text());
    ui->timerDuration->setVisible(true);
    ui->timerSet->setVisible(false);
    ui->startstopTimer->setChecked(true);
    ui->startstopTimer->clicked(true);
}

void coretime::on_startstopTimer_clicked(bool checked)
{
    qDebug() << "Timer state " << checked;

    if (ui->timerText->text() == "00:00:00") {
        return;
    }

    if (checked) {
        ui->startstopTimer->setText("Pause");
        forTimer->start(1000);
    } else {
        ui->startstopTimer->setText("Start");
        forTimer->stop();
    }
}

void coretime::on_resetTimer_clicked()
{
    forTimer->stop();

    ui->timerText->setText("00:00:00");
    ui->startstopTimer->setChecked(false);
    ui->startstopTimer->setText("Start");
    ui->timerSet->setVisible(true);
    ui->timerDuration->setVisible(false);
}
//======================Timer=======================End===================================

void coretime::pageClick(QToolButton *btn, int i, QString title)
{
    // all button checked false
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        b->setChecked(false);
    }

    btn->setChecked(true);
    ui->selectedsection->setText(title);
    ui->pages->setCurrentIndex(i);
}

void coretime::on_clock_clicked()
{
    pageClick(ui->clock, 0, tr("Clock"));

    this->setWindowTitle("Clock - CoreTime");
}

void coretime::on_alarm_clicked()
{
    pageClick(ui->alarm, 1, tr("Alarm/Reminders"));

    this->setWindowTitle("Alarm - CoreTime");
}

void coretime::on_stopwatch_clicked()
{
    pageClick(ui->stopwatch, 2, tr("Stopwatch"));

    this->setWindowTitle("Stopwatch - CoreTime");
}

void coretime::on_timer_clicked()
{
    pageClick(ui->timer, 3, tr("Timer"));

    this->setWindowTitle("Timer - CoreTime");
}

void coretime::on_appTitle_clicked()
{
    if (sideView && touch) {
        // all button without text
        for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
            b->setToolButtonStyle(Qt::ToolButtonIconOnly);
            b->setFixedWidth(sideViewIconSize.width() + 10);
        }

        ui->sideView->setFixedWidth(sideViewIconSize.width() + 10);
        sideView = 0;
    } else {
        // all button with text
        for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
            b->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
            b->setFixedWidth(sideViewSize);
        }

        ui->sideView->setFixedWidth(sideViewSize);
        sideView = 1;
    }

}
